@@ -1,8 +1,8 @@
-# -*- coding: ascii -*-
+# -*- coding: utf-8 -*-
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2007 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -14,7 +14,6 @@
 #
 
 import os
-import gc
 
 if 'DISPLAY' in os.environ:
     import gtk
@@ -22,27 +21,31 @@ if 'DISPLAY' in os.environ:
     import pango
     import cairo
 
-import pix
+import ninix.pix
 
 
 class Balloon:
 
-    def __init__(self, sakura, debug=0):
-        self.sakura = sakura
+    def __init__(self, sakura, prefs, debug=0):
+        self.__sakura = sakura
+        self.prefs = prefs
         self.debug = debug
         self.synchronized = []
         self.__scalling = 0
         self.__scale = 100 # %
         self.__use_pna = 0
         self.__alpha_channel = None
-        self.font_name = 'Sans'
+        self.user_interaction = False
         self.window = []
         # create communicatebox
-        self.communicatebox = CommunicateBox(sakura, debug)
+        self.communicatebox = CommunicateBox(self, sakura, debug)
         # create teachbox
-        self.teachbox = TeachBox(sakura, debug)
+        self.teachbox = TeachBox(self, sakura, debug)
         # create inputbox
-        self.inputbox = InputBox(sakura, debug)
+        self.inputbox = InputBox(self, sakura, debug)
+
+    def reset_user_interaction(self):
+        self.user_interaction = False
 
     def get_text_count(self, side):
         if len(self.window) > side:
@@ -98,21 +101,25 @@ class Balloon:
             balloon_window.set_scalling(flag)
 
     def create_gtk_window(self, title):
-        window = gtk.Window()
-        pix.set_rgba_colormap(window)
+        window = ninix.pix.TransparentWindow()
         window.set_title(title)
         window.set_decorated(False)
         window.set_resizable(False)
         window.set_skip_pager_hint(False)
         window.set_skip_taskbar_hint(True)
-        window.set_focus_on_map(False)
-        window.font_name = self.font_name
+        window.font_name = self.prefs.get('balloon_fonts')
         window.connect('delete_event', self.delete)
         window.realize()
         return window
 
+    def identify_window(self, win):
+        for balloon_window in self.window:
+            if win == balloon_window.window.window:
+                return True
+        return False
+
     def delete(self, window, event):
-        self.sakura.ghost.finalize() ## FIXME
+        self.__sakura.finalize()
         return False
 
     def finalize(self):
@@ -158,7 +165,7 @@ class Balloon:
                                                ('kero', 1, 'k%d', balloon1)]:
             gtk_window = self.create_gtk_window(''.join(('balloon.', name)))
             balloon_window = BalloonWindow(
-                gtk_window, side, self.sakura, desc, balloon,
+                gtk_window, side, self.__sakura, desc, balloon,
                 id_format, self.__use_pna, self.__alpha_channel, self.debug)
             self.window.append(balloon_window)
         for balloon_window in self.window:
@@ -177,19 +184,14 @@ class Balloon:
         id_format = 'k%d'
         balloon = self.balloon1 ## FIXME
         balloon_window = BalloonWindow(
-            gtk_window, side, self.sakura, self.desc, balloon,
+            gtk_window, side, self.__sakura, self.desc, balloon,
             id_format, self.__use_pna, self.__alpha_channel, self.debug)
         self.window.append(balloon_window)
         balloon_window.set_scale(self.__scale)
         balloon_window.set_balloon(0) ## FIXME
 
-    def get_balloon_fonts(self):
-        return self.font_name
-
-    def set_balloon_fonts(self, font_name):
-        if self.font_name == font_name:
-            return
-        self.font_name = font_name
+    def reset_fonts(self):
+        font_name = self.prefs.get('balloon_fonts')
         for window in self.window:
             window.window.font_name = font_name
             window.update_gc()
@@ -204,14 +206,18 @@ class Balloon:
         else:
             return (0, 0)
 
+    def reset_balloon(self):
+        scale = scale # %
+        use_pna = 0
+        for balloon_window in self.window:
+            balloon_window.set_use_pna(use_pna)
+            balloon_window.set_scale(scale)
+            balloon_window.reset_balloon()
+
     def reset_balloon_all(self):
         for side in range(len(self.window)):
             self.window[side].reset_balloon()
 
-    def reset_balloon(self, side):
-        if len(self.window) > side:
-            self.window[side].reset_balloon()
-
     def set_balloon_default(self):
         for side in range(len(self.window)):
             self.window[side].set_balloon(0) ## FIXME
@@ -354,16 +360,23 @@ class Balloon:
     def hide_sstp_message(self):
         self.window[0].hide_sstp_message()
 
-    def show_communicatebox(self):
-        self.communicatebox.show()
+    def open_communicatebox(self):
+        if not self.user_interaction:
+            self.user_interaction = True
+            self.communicatebox.show()
 
-    def show_teachbox(self):
-        self.teachbox.show()
+    def open_teachbox(self):
+        if not self.user_interaction:
+            self.user_interaction = True
+            self.__sakura.notify_event('OnTeachStart')
+            self.teachbox.show()
 
-    def show_inputbox(self, symbol, limittime, default):
-        self.inputbox.set_symbol(symbol)
-        self.inputbox.set_limittime(limittime)
-        self.inputbox.show(default)
+    def open_inputbox(self, symbol, limittime=-1, default=None):
+        if not self.user_interaction:
+            self.user_interaction = True
+            self.inputbox.set_symbol(symbol)
+            self.inputbox.set_limittime(limittime)
+            self.inputbox.show(default)
 
 class BalloonWindow:
 
@@ -371,7 +384,7 @@ class BalloonWindow:
                  id_format, use_pna, alpha, debug):
         self.window = window
         self.side = side
-        self.sakura = sakura
+        self.__sakura = sakura
         self.desc = desc
         self.balloon = balloon
         self.balloon_id = None
@@ -390,11 +403,12 @@ class BalloonWindow:
         self.__use_pna = use_pna
         self.__alpha_channel = alpha
         self.text_count = 0
+        self.layout = None
         # load files
         self.pixbuf = {}
         for key, (path, config) in balloon.iteritems():
             try:
-                pixbuf = pix.create_pixbuf_from_file(path)
+                pixbuf = ninix.pix.create_pixbuf_from_file(path)
             except:
                 continue
             self.pixbuf[key] = (pixbuf,
@@ -436,11 +450,10 @@ class BalloonWindow:
 
     def reset_pixbuf_cache(self):
         self.pixbuf = None
-        gc.collect()
         self.pixbuf = {}
         for key, (path, config) in self.balloon.iteritems():
             try:
-                pixbuf = pix.create_pixbuf_from_file(path,
+                pixbuf = ninix.pix.create_pixbuf_from_file(path,
                                                      use_pna=self.__use_pna)
             except:
                 continue
@@ -502,33 +515,36 @@ class BalloonWindow:
             x = self.config_adjust('sstpmessage.x', w,  50)
             y = self.config_adjust('sstpmessage.y', h, -20)
             self.sstp.append((x, y)) # sstp message
-        # arrow pixmaps and masks
+        # arrow pixbufs and masks
         x, y = self.arrow[0]
         pixbuf, (w, h) = self.pixbuf['arrow0']
         scale = self.get_scale()
         w = max(1, int(w * scale / 100))
         h = max(1, int(h * scale / 100))
-        pixmap, mask = pixbuf.scale_simple(
-            w, h, gtk.gdk.INTERP_BILINEAR).render_pixmap_and_mask(255)
-        self.arrow0_pixmap = pixmap, mask, (w, h)
+        mask = gtk.gdk.Pixmap(None, w, h, 1)
+        pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
+        pixbuf.render_threshold_alpha(mask, 0, 0, 0, 0, w, h, 255)
+        self.arrow0_pixbuf = pixbuf, mask, (w, h)
         self.arrow0_gc = self.new_mask_gc(mask, x, y)
         x, y = self.arrow[1]
         pixbuf, (w, h) = self.pixbuf['arrow1']
         w = max(1, int(w * scale / 100))
         h = max(1, int(h * scale / 100))
-        pixmap, mask = pixbuf.scale_simple(
-            w, h, gtk.gdk.INTERP_BILINEAR).render_pixmap_and_mask(255)
-        self.arrow1_pixmap = pixmap, mask, (w, h)
+        mask = gtk.gdk.Pixmap(None, w, h, 1)
+        pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
+        pixbuf.render_threshold_alpha(mask, 0, 0, 0, 0, w, h, 255)
+        self.arrow1_pixbuf = pixbuf, mask, (w, h)
         self.arrow1_gc = self.new_mask_gc(mask, x, y)
-        # sstp marker pixmap and mask
+        # sstp marker pixbuf and mask
         if self.side == 0 and 'sstp' in self.pixbuf:
             x, y = self.sstp[0]
             pixbuf, (w, h) = self.pixbuf['sstp']
             w = max(1, int(w * scale / 100))
             h = max(1, int(h * scale / 100))
-            pixmap, mask = pixbuf.scale_simple(
-                w, h, gtk.gdk.INTERP_BILINEAR).render_pixmap_and_mask(255)
-            self.sstp_pixmap = pixmap, mask, (w, h)
+            mask = gtk.gdk.Pixmap(None, w, h, 1)
+            pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
+            pixbuf.render_threshold_alpha(mask, 0, 0, 0, 0, w, h, 255)
+            self.sstp_pixbuf = pixbuf, mask, (w, h)
             self.sstp_gc = self.new_mask_gc(mask, x, y)
         # font
         default_size = 12 # for Windows environment
@@ -644,21 +660,26 @@ class BalloonWindow:
         if balloon_id not in self.pixbuf:
             balloon_id = self.id_format % (0 + self.direction)
         self.balloon_id = balloon_id
-        # change pixmap and window position
+        # change pixbuf and window position
         x, y = self.position
         pixbuf, (w, h) = self.pixbuf[balloon_id]
         scale = self.get_scale()
-        self.width  = max(1, int(w * scale / 100))
+        self.width = max(1, int(w * scale / 100))
         self.height = max(1, int(h * scale / 100))
         self.darea.set_size_request(self.width, self.height)
         self.balloon_pixbuf = pixbuf.scale_simple(
             self.width, self.height, gtk.gdk.INTERP_BILINEAR)
-        pixmap, mask = self.balloon_pixbuf.render_pixmap_and_mask(255)
-        self.window.shape_combine_mask(mask, 0, 0)
+        mask = gtk.gdk.Pixmap(None, self.width, self.height, 1)
+        self.balloon_pixbuf.render_threshold_alpha(
+            mask, 0, 0, 0, 0, self.width, self.height, 255)
+        if self.window.is_composited():
+            self.window.input_shape_combine_mask(mask, 0, 0)
+        else:
+            self.window.shape_combine_mask(mask, 0, 0)
         if self.__shown:
             self.update_gc()
         if reset_position:
-            self.sakura.ghost.position_balloons() ## FIXME
+            self.__sakura.position_balloons()
 
     def set_direction(self, dir):
         if self.direction != dir:
@@ -714,11 +735,11 @@ class BalloonWindow:
     def show(self):
         if not self.__shown:
             self.window.show()
+            self.__shown = 1
             # make sure window is in its position
             self.__move()
             self.update_gc()
             self.raise_()
-            self.__shown = 1
 
     def hide(self):
         if self.__shown:
@@ -765,10 +786,10 @@ class BalloonWindow:
         # draw/erase sstp marker
         if 'sstp' in self.pixbuf:
             x, y = self.sstp[0]
-            w, h = self.sstp_pixmap[2]
+            w, h = self.sstp_pixbuf[2]
             if self.sstp_message:
-                self.darea.window.draw_drawable(
-                    self.sstp_gc, self.sstp_pixmap[0], 0, 0, x, y, w, h)
+                self.darea.window.draw_pixbuf(
+                    self.sstp_gc, self.sstp_pixbuf[0], 0, 0, x, y, w, h)
             else:
                 self.redraw_area(x, y, w, h)
         # draw/erase sstp message
@@ -782,20 +803,20 @@ class BalloonWindow:
 
     def redraw_arrow0(self, clear=1):
         x, y = self.arrow[0]
-        w, h = self.arrow0_pixmap[2]
+        w, h = self.arrow0_pixbuf[2]
         if self.lineno > 0:
-            self.darea.window.draw_drawable(
-                self.arrow0_gc, self.arrow0_pixmap[0], 0, 0, x, y, w, h)
+            self.darea.window.draw_pixbuf(
+                self.arrow0_gc, self.arrow0_pixbuf[0], 0, 0, x, y, w, h)
         elif clear:
             self.redraw_area(x, y, w, h)
 
     def redraw_arrow1(self, clear=1):
         x, y = self.arrow[1]
-        w, h = self.arrow1_pixmap[2]
+        w, h = self.arrow1_pixbuf[2]
         if self.lineno + self.lines < len(self.text_buffer) or \
-           self.sakura.script_mode == self.sakura.PAUSE_MODE:
-            self.darea.window.draw_drawable(
-                self.arrow1_gc, self.arrow1_pixmap[0], 0, 0, x, y, w, h)
+           self.__sakura.is_paused():
+            self.darea.window.draw_pixbuf(
+                self.arrow1_gc, self.arrow1_pixbuf[0], 0, 0, x, y, w, h)
         elif clear:
             self.redraw_area(x, y, w, h)
 
@@ -869,12 +890,16 @@ class BalloonWindow:
                     y = int(y)
                 except:
                     continue
-            pixmap, mask = pixbuf.scale_simple(
-                w, h, gtk.gdk.INTERP_BILINEAR).render_pixmap_and_mask(255)
-            gc = pixmap.new_gc()
+            #pixmap, mask = pixbuf.scale_simple(
+            #    w, h, gtk.gdk.INTERP_BILINEAR).render_pixmap_and_mask(255)
+            mask = gtk.gdk.Pixmap(None, w, h, 1)
+            pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
+            pixbuf.render_threshold_alpha(mask, 0, 0, 0, 0, w, h, 255)
+            #gc = pixmap.new_gc()
+            gc = darea.window.new_gc()
             gc.set_clip_mask(mask)
             gc.set_clip_origin(x, y)
-            darea.window.draw_drawable(gc, pixmap, 0, 0, x, y, w, h)
+            darea.window.draw_pixbuf(gc, pixbuf, 0, 0, x, y, w, h)
         # draw text
         i = self.lineno
         j = len(self.text_buffer)
@@ -897,15 +922,15 @@ class BalloonWindow:
                                          x, y, self.layout)
             for l, c in self.sstp_marker:
                 if l == i:
-                    mw, mh = self.sstp_pixmap[2]
+                    mw, mh = self.sstp_pixbuf[2]
                     self.layout.set_text(self.text_buffer[i][:c])
                     text_w, text_h = self.layout.get_pixel_size()
                     mx = x + text_w
                     my = y + (self.font_height + self.line_space) / 2
                     my = my - mh / 2
-                    gc = self.new_mask_gc(self.sstp_pixmap[1], mx, my)
-                    self.darea.window.draw_drawable(
-                        gc, self.sstp_pixmap[0], 0, 0, mx, my, mw, mh)
+                    gc = self.new_mask_gc(self.sstp_pixbuf[1], mx, my)
+                    self.darea.window.draw_pixbuf(
+                        gc, self.sstp_pixbuf[0], 0, 0, mx, my, mw, mh)
             i += 1
             line += 1
         if self.side == 0 and self.sstp_message:
@@ -1009,11 +1034,11 @@ class BalloonWindow:
             if self.selection != new_selection:
                 sl, sn, el, en, link_id, raw_text, text = \
                     self.link_buffer[new_selection]
-                self.sakura.notify_event(
+                self.__sakura.notify_event(
                     'OnChoiceEnter', raw_text, link_id, self.selection)
         else:
             if self.selection is not None:
-                self.sakura.notify_event('OnChoiceEnter')
+                self.__sakura.notify_event('OnChoiceEnter')
         if new_selection == self.selection:
             return 0
         else:
@@ -1053,19 +1078,19 @@ class BalloonWindow:
         return True
 
     def button_press(self, darea, event):
-        self.sakura.reset_idle_time()
+        self.__sakura.reset_idle_time()
         if event.type == gtk.gdk.BUTTON_PRESS:
             click = 1
         else:
             click = 2
-        if self.sakura.script_mode == self.sakura.PAUSE_MODE:
-            self.sakura.notify_balloon_click(event.button, click, self.side)
+        if self.__sakura.is_paused():
+            self.__sakura.notify_balloon_click(event.button, click, self.side)
             return True
         # arrows
         px = int(event.x)
         py = int(event.y)
         # up arrow
-        w, h = self.arrow0_pixmap[2]
+        w, h = self.arrow0_pixbuf[2]
         x, y = self.arrow[0]
         if x <= px <= x + w and y <= py <= y + h:
             if self.lineno > 0:
@@ -1073,7 +1098,7 @@ class BalloonWindow:
                 self.redraw()
             return True
         # down arrow
-        w, h = self.arrow1_pixmap[2]
+        w, h = self.arrow1_pixbuf[2]
         x, y = self.arrow[1]
         if x <= px <= x + w and y <= py <= y + h:
             if self.lineno + self.lines < len(self.text_buffer):
@@ -1085,10 +1110,11 @@ class BalloonWindow:
         if self.selection is not None:
             sl, sn, el, en, link_id, raw_text, text = \
                 self.link_buffer[self.selection]
-            self.sakura.notify_link_selection(link_id, raw_text, self.selection)
+            self.__sakura.notify_link_selection(
+                link_id, raw_text, self.selection)
             return True
         # balloon's background
-        self.sakura.notify_balloon_click(event.button, click, self.side)
+        self.__sakura.notify_balloon_click(event.button, click, self.side)
         return True
 
     def clear_text(self):
@@ -1224,7 +1250,7 @@ class BalloonWindow:
 
     def append_image(self, path, x, y):
         try:
-            pixbuf = pix.create_pixbuf_from_file(path)
+            pixbuf = ninix.pix.create_pixbuf_from_file(path)
         except:
             return
         self.show()
@@ -1232,7 +1258,6 @@ class BalloonWindow:
         h = pixbuf.get_height()
         self.images.append((pixbuf, (w, h), (x, y)))
         del pixbuf
-        gc.collect()
         self.redraw()
 
     def draw_last_line(self, column=0):
@@ -1249,15 +1274,15 @@ class BalloonWindow:
                 self.darea.queue_draw_area(x, y, w, h)
             for l, c in self.sstp_marker:
                 if l == line:
-                    mw, mh = self.sstp_pixmap[2]
+                    mw, mh = self.sstp_pixbuf[2]
                     self.layout.set_text(self.text_buffer[l][:c])
                     text_w, text_h = self.layout.get_pixel_size()
                     mx = x + text_w
                     my = y + (self.font_height + self.line_space) / 2
                     my = my - mh / 2
-                    gc = self.new_mask_gc(self.sstp_pixmap[1], mx, my)
-                    self.darea.window.draw_drawable(
-                        gc, self.sstp_pixmap[0], 0, 0, mx, my, mw, mh)
+                    gc = self.new_mask_gc(self.sstp_pixbuf[1], mx, my)
+                    self.darea.window.draw_pixbuf(
+                        gc, self.sstp_pixbuf[0], 0, 0, mx, my, mw, mh)
         else:
             self.redraw()
             while line >= self.lineno + self.lines:
@@ -1269,8 +1294,9 @@ class CommunicateWindow:
     NAME = ''
     ENTRY = ''
 
-    def __init__(self, sakura, debug):
-        self.sakura = sakura
+    def __init__(self, balloon, sakura, debug):
+        self.__balloon = balloon
+        self.__sakura = sakura
         self.debug = debug
         self.window = None
         self.dragged = 0
@@ -1300,7 +1326,7 @@ class CommunicateWindow:
             path, config = balloon
             # load pixmap
             try:
-                image, mask = pix.create_pixmap_from_file(path)
+                image, mask = ninix.pix.create_pixmap_from_file(path)
             except:
                 image, mask = None, None
         if image is not None:
@@ -1379,14 +1405,14 @@ class CommunicateBox(CommunicateWindow):
     def delete(self, widget, event):
         self.window.hide()
         self.cancel()
-        self.sakura.user_interaction = 0
+        self.__balloon.reset_user_interaction()
         return True
 
     def key_press(self, widget, event):
         if event.keyval == gtk.keysyms.Escape:
             self.window.hide()
             self.cancel()
-            self.sakura.user_interaction = 0
+            self.__balloon.reset_user_interaction()
             return True
         return False
 
@@ -1404,7 +1430,8 @@ class CommunicateBox(CommunicateWindow):
     def send(self, data):
         if data:
             data = unicode(data, 'utf-8')
-        self.sakura.notify_user_communicate(data)
+        if data is not None:
+            self.__sakura.notify_event('OnCommunicate', 'user', data)
 
 class TeachBox(CommunicateWindow):
 
@@ -1420,7 +1447,8 @@ class TeachBox(CommunicateWindow):
     def send(self, data):
         if data:
             data = unicode(data, 'utf-8')
-        self.sakura.notify_user_teach(data)
+        self.__sakura.notify_user_teach(data)
+        self.__balloon.reset_user_interaction()
 
 class InputBox(CommunicateWindow):
 
@@ -1471,7 +1499,17 @@ class InputBox(CommunicateWindow):
             gobject.source_remove(self.timeout_id)            
         if data:
             data = unicode(data, 'utf-8')
-        self.sakura.notify_user_input(self.symbol, data)
+        if data is None:
+            data = ''
+        ## CHECK: symbol
+        if self.symbol == 'OnUserInput' and \
+           self.__sakura.notify_event('OnUserInput', data):
+            pass
+        elif self.__sakura.notify_event('OnUserInput', self.symbol, data):
+            pass
+        elif self.__sakura.notify_event(self.symbol, data):
+            pass
+        self.__balloon.reset_user_interaction()
 
 
 def test():
