@@ -1,7 +1,7 @@
-# -*- coding: ascii -*-
+# -*- coding: utf-8 -*-
 #
 #  bln.py - a easyballoon compatible Saori module for ninix
-#  Copyright (C) 2002-2007 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -20,7 +20,6 @@ import random
 import math
 import time
 import sys
-import gc
 
 if 'DISPLAY' in os.environ:
     if 'gtk' not in sys.modules:
@@ -45,13 +44,13 @@ class Saori(SAORI):
     def __init__(self):
         SAORI.__init__(self)
         self.blns = {}
-        self.__ghost = None
+        self.__sakura = None
 
-    def need_ghost_backdoor(self, ghost):
-        self.__ghost = ghost
+    def need_ghost_backdoor(self, sakura):
+        self.__sakura = sakura
 
     def check_import(self):
-        if self.__ghost and gtk:
+        if self.__sakura and gtk:
             return 1
         else:
             return 0
@@ -141,7 +140,7 @@ class Saori(SAORI):
                 del bln[bln_id]
             if text:
                 if update == 0 or bln_id not in bln:
-                    bln[bln_id] = Balloon(self.__ghost, self.dir,
+                    bln[bln_id] = Balloon(self.__sakura, self.prefs, self.dir,
                                           data, text,
                                           offset_x, offset_y, name, bln_id)
                 else:
@@ -159,10 +158,11 @@ class Saori(SAORI):
 
 class Balloon:
 
-    def __init__(self, ghost, dir, data,
+    def __init__(self, sakura, prefs, dir, data,
                  text, offset_x, offset_y, name, bln_id):
         self.dir = dir
-        self.__ghost = ghost
+        self.__sakura = sakura
+        self.prefs = prefs
         self.dragged = 0
         self.name = name
         self.id = bln_id
@@ -194,18 +194,16 @@ class Balloon:
                                gtk.gdk.BUTTON_RELEASE_MASK|
                                gtk.gdk.POINTER_MOTION_MASK|
                                gtk.gdk.LEAVE_NOTIFY_MASK)
-        scrn_w = gtk.gdk.screen_width()
-        scrn_h = gtk.gdk.screen_height()
-        scrn_h = scrn_h - self.get_bottom_margin()
+        left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
         s0_x, s0_y, s0_w, s0_h = self.get_sakura_status('SurfaceSakura')
         s1_x, s1_y, s1_w, s1_h = self.get_sakura_status('SurfaceKero')
         b0_x, b0_y, b0_w, b0_h = self.get_sakura_status('BalloonSakura')
         b1_x, b1_y, b1_w, b1_h = self.get_sakura_status('BalloonKero')
-        if s0_x + s0_w / 2 > scrn_w / 2:
+        if s0_x + s0_w / 2 > left + scrn_w / 2:
             self.direction0 = 0 # left
         else:
             self.direction0 = 1 # right
-        if s1_x + s1_w / 2 > scrn_w / 2:
+        if s1_x + s1_w / 2 > left + scrn_w / 2:
             self.direction1 = 0 # left
         else:
             self.direction1 = 1 # right  
@@ -229,8 +227,8 @@ class Balloon:
         except:
             self.destroy()
             return
-        self.x = 0
-        self.y = 0 + self.get_top_margin()
+        self.x = left
+        self.y = top
         self.scale = self.get_sakura_status('SurfaceScale') ## FIXME
         w = balloon_pixbuf.get_width()
         h = balloon_pixbuf.get_height()
@@ -241,30 +239,28 @@ class Balloon:
                 w, h, gtk.gdk.INTERP_BILINEAR)
         balloon_pixmap, mask_pixmap = balloon_pixbuf.render_pixmap_and_mask(1)
         del balloon_pixbuf
-        gc.collect()
         if self.position == 'lefttop':
             pass
         elif self.position == 'leftbottom':
-            self.y = scrn_h - h
+            self.y = top + scrn_h - h
         elif self.position == 'righttop':
-            self.x = scrn_w - w
+            self.x = left + scrn_w - w
         elif self.position == 'rightbottom':
-            self.x = scrn_w - w
-            self.y = scrn_h - h
+            self.x = left + scrn_w - w
+            self.y = top + scrn_h - h
         elif self.position == 'center':
-            self.x = (scrn_w - w) / 2
-            self.y = (scrn_h - h) / 2
+            self.x = left + (scrn_w - w) / 2
+            self.y = top + (scrn_h - h) / 2
         elif self.position == 'leftcenter':
-            self.x = 0
-            self.y = (scrn_h - h) / 2
+            self.y = top + (scrn_h - h) / 2
         elif self.position == 'rightcenter':
-            self.x = scrn_w -w
-            self.y = (scrn_h - h) / 2
+            self.x = left + scrn_w -w
+            self.y = top + (scrn_h - h) / 2
         elif self.position == 'centertop':
-            self.x = (scrn_w - w) / 2
+            self.x = left + (scrn_w - w) / 2
         elif self.position == 'centerbottom':
-            self.x = (scrn_w - w) / 2
-            self.y = scrn_h - h
+            self.x = left + (scrn_w - w) / 2
+            self.y = top + scrn_h - h
         elif self.position == 'sakura':
             if self.direction0: # right
                 self.x = s0_x + s0_w
@@ -465,78 +461,72 @@ class Balloon:
     def get_sakura_is_talking(self):
         talking = 0
         try:
-            if self.__ghost.is_talking():
+            if self.__sakura.is_talking():
                 talking = 1
         except:
             pass
         return talking
 
-    def get_top_margin(self):
-        return self.__ghost.get_top_margin()
-
-    def get_bottom_margin(self):
-        return self.__ghost.get_bottom_margin()
-
-    def get_sakura_status(self, key): # FIXME
+    def get_sakura_status(self, key):
         if key == 'SurfaceScale':
-            return self.__ghost.get_surface_scale()
+            return self.prefs.get('surface_scale')
         elif key == 'SurfaceSakura_Shown':
-            if self.__ghost.surface_is_shown(0):
+            if self.__sakura.surface_is_shown(0):
                 s0_shown = 1
             else:
                 s0_shown = 0
             return s0_shown
         elif key == 'SurfaceSakura':
             try:
-                s0_x, s0_y = self.__ghost.get_surface_position(0)
-                s0_w, s0_h = self.__ghost.get_surface_size(0)
+                s0_x, s0_y = self.__sakura.get_surface_position(0)
+                s0_w, s0_h = self.__sakura.get_surface_size(0)
             except:
                 s0_x, s0_y = 0, 0
                 s0_w, s0_h = 0, 0
             return s0_x, s0_y, s0_w, s0_h
         elif key == 'SurfaceKero_Shown':
-            if self.__ghost.surface_is_shown(1):
+            if self.__sakura.surface_is_shown(1):
                 s1_shown = 1
             else:
                 s1_shown = 0
             return s1_shown
         elif key == 'SurfaceKero':
             try:
-                s1_x, s1_y = self.__ghost.get_surface_position(1)
-                s1_w, s1_h = self.__ghost.get_surface_size(1)
+                s1_x, s1_y = self.__sakura.get_surface_position(1)
+                s1_w, s1_h = self.__sakura.get_surface_size(1)
             except:
                 s1_x, s1_y = 0, 0
                 s1_w, s1_h = 0, 0
             return s1_x, s1_y, s1_w, s1_h
         elif key == 'BalloonScale':
-            if self.__ghost.get_balloon_scalling():
-                return self.__ghost.get_surface_scale()
+            if self.prefs.get('balloon_scalling'):
+                return self.prefs.get('surface_scale')
             else:
                 return 100
         elif key == 'BalloonSakura_Shown':
-            if self.__ghost.balloon_is_shown(0):
+            if self.__sakura.balloon_is_shown(0):
                 b0_shown = 1
             else:
                 b0_shown = 0
             return b0_shown
         elif key == 'BalloonSakura':
             try:
-                b0_x, b0_y = self.__ghost.get_balloon_position(0)
-                b0_w, b0_h = self.__ghost.get_balloon_size(0)
+                b0_x, b0_y = self.__sakura.get_balloon_position(0)
+                b0_w, b0_h = self.__sakura.get_balloon_size(0)
             except:
                 b0_x, b0_y = 0, 0
                 b0_w, b0_h = 0, 0
             return b0_x, b0_y, b0_w, b0_h
         elif key == 'BalloonKero_Shown':
-            if self.__ghost.balloon_is_shown(1):
+            if self.__sakura.balloon_is_shown(1):
                 b1_shown = 1
             else:
                 b1_shown = 0
             return b1_shown
         elif key == 'BalloonKero':
             try:
-                b1_x, b1_y = self.__ghost.get_balloon_position(1)
-                b1_w, b1_h = self.__ghost.get_balloon_size(1)
+                b1_x, b1_y = self.__sakura.get_balloon_position(1)
+                b1_w, b1_h = self.__sakura.get_balloon_size(1)
             except:
                 b1_x, b1_y = 0, 0
                 b1_w, b1_h = 0, 0
@@ -720,8 +710,8 @@ class Balloon:
             else:
                 x = int(event.x)
                 y = int(event.y)
-            self.__ghost.notify_event('OnEBMouseDoubleClick', self.name,
-                                      x, y, self.id)
+            self.__sakura.notify_event('OnEBMouseDoubleClick', self.name,
+                                       x, y, self.id)
         return True
 
     def button_release(self, window, event):
@@ -737,11 +727,11 @@ class Balloon:
             y = int(event.y)
         if event.type == gtk.gdk.BUTTON_RELEASE:
             if event.button == 1:
-                self.__ghost.notify_event('OnEBMouseClick', self.name,
-                                          x, y, self.id, 0)
+                self.__sakura.notify_event('OnEBMouseClick', self.name,
+                                           x, y, self.id, 0)
             elif event.button == 3:
-                self.__ghost.notify_event('OnEBMouseClick', self.name,
-                                          x, y, self.id, 1)
+                self.__sakura.notify_event('OnEBMouseClick', self.name,
+                                           x, y, self.id, 1)
         if self.clickerase == 'on':
             self.destroy()
         return True
@@ -769,8 +759,8 @@ class Balloon:
             else:
                 x = int(event.x)
                 y = int(event.y)
-            self.__ghost.notify_event('OnEBMouseMove', self.name,
-                                      x, y, self.id)
+            self.__sakura.notify_event('OnEBMouseMove', self.name,
+                                       x, y, self.id)
             self.move_notify_time = time.time()
         return True
 
