@@ -1,7 +1,7 @@
-# -*- coding: ascii -*-
+# -*- coding: utf-8 -*-
 #
 #  mciaudio.py - a MCIAUDIO compatible Saori module for ninix
-#  Copyright (C) 2002-2005 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -11,8 +11,6 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-#  $Id: mciaudio.py,v 1.10 2005/09/26 04:18:33 shy Exp $
-#
 
 import os
 import sys
@@ -27,14 +25,14 @@ class Saori(SAORI):
         SAORI.__init__(self)
         self.pid = 0
         self.filepath = None
-        self.__ghost = None
+        self.__sakura = None
         self.pause = 0
 
-    def need_ghost_backdoor(self, ghost):
-        self.__ghost = ghost
+    def need_ghost_backdoor(self, sakura):
+        self.__sakura = sakura
 
     def check_import(self):
-        if self.__ghost is not None:
+        if self.__sakura is not None:
             return 1
         else:
             return 0
@@ -79,7 +77,7 @@ class Saori(SAORI):
                     self.pid = os.fork()
                     if self.pid == 0:
                         os.setsid()
-                        for regex, command in self.__ghost.get_helpers():
+                        for regex, command in self.prefs.get_helpers_regex():
                             if regex.search(self.filepath):
                                 self.execute_command(command, self.filepath)
                         os._exit(0)
@@ -95,7 +93,7 @@ class Saori(SAORI):
                 filename = argv[1].replace('\\', '/').lower()
                 if os.path.isabs(filename):
                     return 'SAORI/1.0 400 Bad Request\r\n\r\n'
-                self.filepath = os.path.join(self.__ghost.get_prefix(),
+                self.filepath = os.path.join(self.__sakura.get_prefix(),
                                              'ghost/master', self.dir,
                                              filename)
         return 'SAORI/1.0 204 No Content\r\n\r\n'
