@@ -1,7 +1,7 @@
-# -*- coding: UTF-8 -*-
+# -*- coding: utf-8 -*-
 #
 #  mciaudior.py - a MCIAUDIOR compatible Saori module for ninix
-#  Copyright (C) 2003-2005 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2003-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by MATSUMURA Namihiko <nie@counterghost.net>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -11,7 +11,6 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-#  $Id: mciaudior.py,v 1.10 2005/09/26 04:18:33 shy Exp $
 
 # TODO:
 # - loop 演奏.
@@ -29,18 +28,8 @@ class Saori(SAORI):
         SAORI.__init__(self)
         self.pid = 0
         self.filepath = None
-        self.__ghost = None
         self.pause = 0
 
-    def need_ghost_backdoor(self, ghost):
-        self.__ghost = ghost
-
-    def check_import(self):
-        if self.__ghost is not None:
-            return 1
-        else:
-            return 0
-
     def finalize(self):
         if self.pid != 0:
             try:
@@ -81,7 +70,7 @@ class Saori(SAORI):
                     self.pid = os.fork()
                     if self.pid == 0:
                         os.setsid()
-                        for regex, command in self.__ghost.get_helpers():
+                        for regex, command in self.prefs.get_helpers_regex():
                             if regex.search(self.filepath):
                                 self.execute_command(command, self.filepath)
                         os._exit(0)
