@@ -1,6 +1,6 @@
-# -*- coding: UTF-8 -*-
+# -*- coding: utf-8 -*-
 #
-#  Copyright (C) 2004-2007 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -48,13 +48,13 @@
 
 import os
 import random
-import gc
 
 if 'DISPLAY' in os.environ:
     import gtk
     import gobject
     import ninix.pix
 
+import ninix.home
 
 class Menu:
 
@@ -124,22 +124,15 @@ class Nekoninni:
         self.__running = 0
         self.skin = None
         self.katochan = None
-        self.top_margin = 0
-        self.bottom_margin = 0
 
     def notify_observer(self, event, args): ## FIXME
-        if event in ['set position', 'set surface',
-                     'set top margin', 'set bottom margin']:
-            if event == 'set top margin':
-                self.top_margin = self.target.get_top_margin()
-            if event == 'set bottom margin':
-                self.bottom_margin = self.target.get_bottom_margin()
+        if event in ['set position', 'set surface']:
             if self.skin is not None:
                 self.skin.set_position()
-            if self.katochan is not None:
+            if self.katochan is not None and self.katochan.loaded:
                 self.katochan.set_position()
-        elif event == 'set scale':
-            scale = self.target.get_surface_scale()
+        elif event == 'set scale': ## FIXME
+            scale = self.target.prefs.get('surface_scale')
             if self.skin is not None:
                 self.skin.set_scale(scale)
             if self.katochan is not None:
@@ -157,9 +150,7 @@ class Nekoninni:
         self.target = target
         self.target.set_observer(self)
         self.accelgroup = gtk.AccelGroup()
-        self.top_margin = self.target.get_top_margin()
-        self.bottom_margin = self.target.get_bottom_margin()
-        scale = self.target.get_surface_scale()
+        scale = self.target.prefs.get('surface_scale')
         self.skin = Skin(self.dir, self.accelgroup, self, scale)
         if self.skin is None:
             return 0
@@ -194,8 +185,7 @@ class Nekoninni:
                 self.katochan.get_target())
         ##print 'KATOCHAN:', args
         ##print 'TARGET:', args[4]
-        sakura = self.target.get_sakura()
-        sakura.notify_event('OnNekodorifObject%s' % event, *args)
+        self.target.notify_event('OnNekodorifObject%s' % event, *args)
 
     def has_katochan(self):
         if self.katochan is not None:
@@ -256,7 +246,8 @@ class Skin:
         else:
             self.omni = 0
         self.window = gtk.Window()
-        ##self.window.set_title(''.join(('surface.', name)))
+        name, top_dir = ninix.home.read_profile_txt(dir) # XXX
+        self.window.set_title(name)
         self.window.set_decorated(False)
         self.window.set_resizable(False)
         self.window.connect('delete_event', self.delete)
@@ -326,13 +317,13 @@ class Skin:
             path = os.path.join(self.dir, 'surface%d.png' % self.id[0])
         try:
             pixbuf = ninix.pix.create_pixbuf_from_file(path)
-            if self.__scale != 100:
-                w = max(1, pixbuf.get_width() * self.__scale / 100)
-                h = max(1, pixbuf.get_height() * self.__scale / 100)
+            scale = self.__scale
+            if scale != 100:
+                w = max(1, pixbuf.get_width() * scale / 100)
+                h = max(1, pixbuf.get_height() * scale / 100)
                 pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
             image, mask = pixbuf.render_pixmap_and_mask(255)
             del pixbuf
-            gc.collect()
         except:
             self.nekoninni.finalize()
             return
@@ -342,15 +333,15 @@ class Skin:
         self.window.shape_combine_mask(mask, 0, 0)
 
     def set_position(self, reset=0):
-        scrn_w = gtk.gdk.screen_width()
-        scrn_h = gtk.gdk.screen_height()
+        left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
         if reset:
-            self.x = 0
-            self.y = scrn_h - self.nekoninni.bottom_margin - self.h
-        self.x = min(scrn_w - self.w, self.x)
-        self.x = max(0, self.x)
-        self.y = min(scrn_h - self.nekoninni.bottom_margin - self.h, self.y)
-        self.y = max(self.nekoninni.top_margin, self.y)
+            self.x = left
+            self.y = top + scrn_h - self.h
+        else:
+            self.x = min(left + scrn_w - self.w, self.x)
+            self.x = max(left, self.x)
+            self.y = min(top + scrn_h - self.h, self.y)
+            self.y = max(top, self.y)
         self.window.move(self.x, self.y)
 
     def move(self, x_delta, y_delta): ## FIXME
@@ -438,6 +429,7 @@ class Katochan:
         self.settings['wave'] = None
         self.settings['wave.loop'] = 0
         self.__scale = 100
+        self.loaded = False
 
     def get_name(self):
         return self.data['name']
@@ -449,11 +441,10 @@ class Katochan:
         return 0 # 0/1 = きのこに当たっていない(ない場合を含む)／当たった
 
     def get_target(self): ## FIXME
-        sakura = self.target.get_sakura()
         if self.side == 0 :
-            return sakura.get_selfname()
+            return self.target.get_selfname()
         else:
-            return sakura.get_keroname()
+            return self.target.get_keroname()
 
     def get_ghost_name(self):
         if 'for' in self.data: # 落下物が主に対象としているゴーストの名前
@@ -509,24 +500,24 @@ class Katochan:
             return
         target_x, target_y = self.target.get_surface_position(self.side)
         target_w, target_h = self.target.get_surface_size(self.side)
-        self.top_margin = self.target.get_top_margin()
-        self.bottom_margin = self.target.get_bottom_margin()
+        scale = self.__scale
+        left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
         self.x = target_x + target_w / 2 - self.w / 2 + \
-                 self.offset_x * self.__scale / 100
-        self.y = self.top_margin + self.offset_y * self.__scale / 100
+                 self.offset_x * scale / 100
+        self.y = top + self.offset_y * scale / 100
         self.window.move(self.x, self.y)
 
     def set_surface(self):
         path = os.path.join(self.data['dir'], 'surface%d.png' % self.id)
         try:
             pixbuf = ninix.pix.create_pixbuf_from_file(path)
-            if self.__scale != 100:
-                w = pixbuf.get_width() * self.__scale / 100
-                h = pixbuf.get_height() * self.__scale / 100
+            scale = self.__scale
+            if scale != 100:
+                w = pixbuf.get_width() * scale / 100
+                h = pixbuf.get_height() * scale / 100
                 pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
             image, mask = pixbuf.render_pixmap_and_mask(255)
             del pixbuf
-            gc.collect()
         except:
             self.nekoninni.finalize()
             return
@@ -538,7 +529,7 @@ class Katochan:
 
     def load(self, data):
         self.data = data
-        self.__scale = self.target.get_surface_scale()
+        self.__scale = self.target.prefs.get('surface_scale')
         self.set_state('before')
         ##print 'DATA:', self.data
         if 'category' in self.data:
@@ -585,9 +576,10 @@ class Katochan:
         self.offset_x = offset_x
         self.offset_y = offset_y
         self.window = gtk.Window()
-        ##self.window.set_title(''.join(('surface.', name)))
+        self.window.set_title(self.data['name'])
         self.window.set_decorated(False)
         self.window.set_resizable(False)
+        self.window.set_skip_taskbar_hint(True) # XXX
         self.window.connect('delete_event', self.delete)
         self.darea = gtk.DrawingArea()
         self.darea.set_events(gtk.gdk.EXPOSURE_MASK)
@@ -599,6 +591,7 @@ class Katochan:
         self.id = 0
         self.set_surface()
         self.set_position()
+        self.loaded = True
 
     def drop(self): ## FIXME
         self.set_state('fall')
@@ -645,10 +638,11 @@ class Katochan:
             return 0
 
     def check_mikire(self):
-        scrn_w = gtk.gdk.screen_width()
-        scrn_h = gtk.gdk.screen_height()
-        if self.x > scrn_w or self.x + self.w < 0 or \
-           self.y > scrn_h or self.y + self.h < 0:
+        left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
+        if self.x + self.w - self.w / 3 > left + scrn_w or \
+                self.x + self.w / 3 < left or \
+                self.y + self.h - self.h / 3 > top + scrn_h or \
+                self.y + self.h / 3 < top:
             return 1
         else:
             return 0
