@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  Copyright (C) 2001-2004 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2004-2007 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -17,7 +17,6 @@ import re
 import sys
 import urllib
 import time
-import gc
 
 from xml.dom import pulldom
 
@@ -499,7 +498,6 @@ class UI:
                     pixbuf = ninix.pix.create_pixbuf_from_file(filename)
                     image, mask = pixbuf.render_pixmap_and_mask(255)
                     del pixbuf
-                    gc.collect()
                 except:
                     darea.window.set_back_pixmap(None, False)
                 else:
@@ -559,7 +557,7 @@ class UI:
         return vbox
 
     def launch_browser(self, url):
-        command = self.ngm.app.get_browser()
+        command = self.ngm.app.prefs.get('browser')
         if '%s' not in command:
             sys.stderr.write('cannot execute command (%s missing)\n')
             return
@@ -723,10 +721,11 @@ class NGM:
 
     def update_current(self): ## FIXME
         item = self.app.find_ghost_by_name(self.get('Name'), 0)
-        ghost = self.app.select_ghost_from_list(item)
-        if not ghost.is_running():
+        sakura = self.app.select_ghost_from_list(item)
+        if not sakura.is_running():
             self.app.start_sakura(item, init=1)
-        ghost.update()
+        sakura.enqueue_script('\![updatebymyself]\e', 'NGM',
+                              None, None, 0, 0, None)
 
 
 if __name__ == '__main__':
